@@ -26,16 +26,14 @@ def get_docker_client():
     """Wrapper around client.from_env() that also pings the daemon
     to make sure it can connect and if not, raises an error."""
     import docker
-    from requests.exceptions import ConnectionError
 
-    client = docker.from_env()
     try:
+        client = docker.from_env()
         client.ping()
-    except ConnectionError as e:
+        return client
+    except Exception as e:
         raise DockerUnavailableError("Could not connect to the Docker daemon") from e
 
-    return client
-
 
 def copy_to_container(container: "Container", source_path: str, target_path: str) -> None:
     """
@@ -75,10 +73,10 @@ def patch_and_save_config(config, patch):
         # Default to creating a config in the user's homedir rather than local.
         config_dir = Path(os.environ["HOME"]) / Path(HOME_SUB_DIR)
         config_path = config_dir / Path(".sgconfig")
-        click.echo("No config file detected, creating one at %s" % config_path)
+        logging.debug("No config file detected, creating one at %s" % config_path)
         config_dir.mkdir(exist_ok=True, parents=True)
     else:
-        click.echo("Updating the existing config file at %s" % config_path)
+        logging.debug("Updating the existing config file at %s" % config_path)
     new_config = patch_config(config, patch)
     overwrite_config(new_config, config_path)
     return str(config_path)
