@@ -37,7 +37,7 @@ DEFAULTS: ConfigDict = {
     "SG_CONFIG_DIR": "",
     "SG_REPO_LOOKUP": "",
     "SG_REPO_LOOKUP_OVERRIDE": "",
-    "SG_S3_HOST": "//localhost",
+    "SG_S3_HOST": "localhost",
     "SG_S3_PORT": "9000",
     "SG_S3_SECURE": "false",
     "SG_S3_BUCKET": "splitgraph",
@@ -54,19 +54,20 @@ DEFAULTS: ConfigDict = {
     "SG_UPDATE_FREQUENCY": "86400",
     "SG_UPDATE_LAST": "0",
     "SG_UPDATE_ANONYMOUS": "false",
+    "SG_PLUGIN_DIR": "",
     # Some default sections: these can't be overridden via envvars.
     "external_handlers": {"S3": "splitgraph.hooks.s3.S3ExternalObjectHandler"},
-    "mount_handlers": {
-        "postgres_fdw": "splitgraph.hooks.mount_handlers.mount_postgres",
-        "mongo_fdw": "splitgraph.hooks.mount_handlers.mount_mongo",
-        "mysql_fdw": "splitgraph.hooks.mount_handlers.mount_mysql",
-        "socrata": "splitgraph.ingestion.socrata.mount.mount_socrata",
-        "elasticsearch": "splitgraph.hooks.mount_handlers.mount_elasticsearch",
+    "data_sources": {
+        "postgres_fdw": "splitgraph.hooks.data_source.PostgreSQLDataSource",
+        "mongo_fdw": "splitgraph.hooks.data_source.MongoDataSource",
+        "mysql_fdw": "splitgraph.hooks.data_source.MySQLDataSource",
+        "socrata": "splitgraph.ingestion.socrata.mount.SocrataDataSource",
+        "elasticsearch": "splitgraph.hooks.data_source.ElasticSearchDataSource",
     },
 }
 
 ALL_KEYS = list(DEFAULTS.keys())
-KEYS = [k for k in ALL_KEYS if k not in ["remotes", "external_handlers", "mount_handlers"]]
+KEYS = [k for k in ALL_KEYS if k not in ["remotes", "external_handlers", "data_sources"]]
 # Keys whose contents we don't print fully
 SENSITIVE_KEY_SUFFIXES = ["_PWD", "_TOKEN"]
 
@@ -164,4 +165,5 @@ This can also be changed by passing `--verbosity` to `sgr`, e.g. `sgr --verbosit
     "SG_UPDATE_FREQUENCY": "How often to check for updates when sgr is run, in seconds. Set to 0 to disable.",
     "SG_UPDATE_LAST": "Last timestamp an update check was performed. Internal.",
     "SG_UPDATE_ANONYMOUS": "Set to `true` to disable sending the user's ID to the update checker.",
+    "SG_PLUGIN_DIR": "Extra directory to look for plugins in. Each subdirectory must have a plugin.py file with a top-level __plugin__ variable pointing at the plugin class",
 }
