@@ -12,9 +12,9 @@ import struct
 from datetime import datetime
 from functools import reduce
 from hashlib import sha256
-from random import getrandbits
-from typing import Any, Dict, List, Optional, Set, Tuple, Union, TYPE_CHECKING, cast
+from typing import Any, Dict, List, Optional, Set, Tuple, Union, TYPE_CHECKING, cast, TypeVar
 
+from psycopg2._json import Json
 from psycopg2.errors import UniqueViolation
 from psycopg2.sql import SQL, Identifier
 from tqdm import tqdm
@@ -26,11 +26,11 @@ from splitgraph.core.indexing.range import (
     filter_range_index,
 )
 from splitgraph.core.metadata_manager import MetadataManager, Object
-from splitgraph.core.types import Changeset, TableSchema
+from splitgraph.core.types import Changeset, TableSchema, Comparable
 from splitgraph.engine import ResultShape
 from splitgraph.engine.postgres.engine import SG_UD_FLAG, add_ud_flag_column, get_change_key
 from splitgraph.exceptions import SplitGraphError
-from .common import adapt, SPLITGRAPH_META_SCHEMA
+from .common import adapt, SPLITGRAPH_META_SCHEMA, get_temporary_table_id
 from .sql import select
 
 if TYPE_CHECKING:
@@ -92,21 +92,23 @@ def get_chunk_groups(chunks: List[Tuple[str, Any, Any]],) -> List[List[Tuple[str
     # within overlap groups.
 
     # no tuple concatenation (typechecker complains)
+
     chunks: List[Tuple[int, str, Any, Any]] = [
         (i, chunk[0], chunk[1], chunk[2]) for i, chunk in enumerate(chunks)
     ]
-
     groups: List[List[Tuple[int, str, Any, Any]]] = []
     current_group: List[Tuple[int, str, Any, Any]] = []
     current_group_start = None
     current_group_end = None
-    for original_id, chunk_id, start, end in sorted(chunks, key=lambda c: c[2]):
+    for original_id, chunk_id, start, end in sorted(chunks, key=lambda c: c[2]):  # type:ignore
         if not current_group:
             current_group = [(original_id, chunk_id, start, end)]
             current_group_start = start
             current_group_end = end
             continue
 
+        assert current_group_start
+        assert current_group_end
         # See if the chunk overlaps with the current chunk group
         if start <= current_group_end and end >= current_group_start:
             current_group.append((original_id, chunk_id, start, end))
@@ -323,7 +325,7 @@ class FragmentManager(MetadataManager):
             pk_index = 0
             row = []
             for col in sorted(table_schema):
-                if col not in pk_cols:
+                if col.name not in pk_cols:
                     row.append(data[1][col.name])
                 else:
                     row.append(pk[pk_index])
@@ -360,7 +362,9 @@ class FragmentManager(MetadataManager):
         # By default (e.g. for changesets where nothing was deleted) we use a 0 hash (since adding it to any other
         # hash has no effect).
         digests = self.object_engine.run_sql(
-            query, [o for row in rows for o in row], return_shape=ResultShape.MANY_ONE
+            query,
+            [o if not isinstance(o, dict) else Json(o) for row in rows for o in row],
+            return_shape=ResultShape.MANY_ONE,
         )
         return (
             reduce(operator.add, map(Digest.from_memoryview, digests), Digest.empty()),
@@ -375,6 +379,7 @@ class FragmentManager(MetadataManager):
         extra_indexes: Optional[ExtraIndexInfo] = None,
         in_fragment_order: Optional[List[str]] = None,
         overwrite: bool = False,
+        table_name: Optional[str] = None,
     ) -> List[str]:
         """
         Store and register multiple changesets as fragments.
@@ -388,6 +393,7 @@ class FragmentManager(MetadataManager):
         """
         object_ids = []
         logging.info("Storing and indexing table %s", table.table_name)
+        table_name = table_name or table.table_name
         for sub_changeset in tqdm(
             changesets, unit="objs", ascii=SG_CMD_ASCII, disable=len(changesets) < 3
         ):
@@ -396,7 +402,9 @@ class FragmentManager(MetadataManager):
             # Store the fragment in a temporary location and then find out its hash and rename to the actual target.
             # Optimisation: in the future, we can hash the upserted rows that we need preemptively and possibly
             # avoid storing the object altogether if it's a duplicate.
-            tmp_object_id = self._store_changeset(sub_changeset, table.table_name, schema)
+            tmp_object_id = self._store_changeset(
+                sub_changeset, table_name, schema, table.table_schema
+            )
 
             (
                 deletion_hash,
@@ -420,12 +428,21 @@ class FragmentManager(MetadataManager):
                         in_fragment_order, table.table_schema
                     )
 
-                self.object_engine.store_object(
-                    object_id=object_id,
-                    source_query=source_query,
-                    schema_spec=add_ud_flag_column(table.table_schema),
-                    overwrite=overwrite,
-                )
+                try:
+                    self.object_engine.store_object(
+                        object_id=object_id,
+                        source_query=source_query,
+                        schema_spec=add_ud_flag_column(table.table_schema),
+                        overwrite=overwrite,
+                    )
+                except UniqueViolation:
+                    # Someone registered this object (perhaps a concurrent pull) already.
+                    logging.info(
+                        "Object %s for table %s/%s already exists, continuing...",
+                        object_id,
+                        table.repository,
+                        table.table_name,
+                    )
                 self.object_engine.delete_table("pg_temp", tmp_object_id)
                 # There are some cases where an object can already exist in the object engine (in the cache)
                 # but has been deleted from the metadata engine, so when it's recreated, we'll skip
@@ -473,12 +490,14 @@ class FragmentManager(MetadataManager):
         object_id = "o" + sha256((content_hash + schema_hash).encode("ascii")).hexdigest()[:-2]
         return deletion_hash, insertion_hash, object_id, rows_inserted, rows_deleted
 
-    def _store_changeset(self, sub_changeset: Any, table: str, schema: str) -> str:
+    def _store_changeset(
+        self, sub_changeset: Any, table: str, schema: str, table_schema: TableSchema
+    ) -> str:
         tmp_object_id = get_temporary_table_id()
         upserted = [pk for pk, data in sub_changeset.items() if data[0]]
         deleted = [pk for pk, data in sub_changeset.items() if not data[0]]
         self.object_engine.store_fragment(
-            upserted, deleted, "pg_temp", tmp_object_id, schema, table
+            upserted, deleted, "pg_temp", tmp_object_id, schema, table, table_schema,
         )
         return tmp_object_id
 
@@ -759,13 +778,22 @@ class FragmentManager(MetadataManager):
                 source_query += SQL(" ") + self._get_order_by_clause(
                     in_fragment_order, table_schema
                 )
-            self.object_engine.store_object(
-                object_id=object_id,
-                source_query=source_query,
-                schema_spec=add_ud_flag_column(table_schema),
-                source_query_args=source_query_args,
-                overwrite=overwrite,
-            )
+            try:
+                self.object_engine.store_object(
+                    object_id=object_id,
+                    source_query=source_query,
+                    schema_spec=add_ud_flag_column(table_schema),
+                    source_query_args=source_query_args,
+                    overwrite=overwrite,
+                )
+            except UniqueViolation:
+                # Someone registered this object (perhaps a concurrent pull) already.
+                logging.info(
+                    "Object %s for table %s/%s already exists, continuing...",
+                    object_id,
+                    source_schema,
+                    source_table,
+                )
         with self.metadata_engine.savepoint("object_register"):
             try:
                 self._register_object(
@@ -1084,8 +1112,3 @@ def _conflate_changes(
                 changeset[change_pk] = (upserted, old_change[1], new_row)
 
     return changeset
-
-
-def get_temporary_table_id() -> str:
-    """Generate a random ID for temporary/staging objects that haven't had their ID calculated yet."""
-    return str.format("sg_tmp_{:032x}", getrandbits(128))
