@@ -5,7 +5,6 @@ from contextlib import contextmanager
 from datetime import datetime
 from random import getrandbits
 from typing import (
-    Any,
     Dict,
     Iterator,
     List,
@@ -28,7 +27,6 @@ from splitgraph.config import (
 )
 from splitgraph.engine import ResultShape
 from splitgraph.exceptions import SplitGraphError, TableNotFoundError
-from splitgraph.hooks.mount_handlers import init_fdw
 from .common import set_tag, manage_audit, set_head
 from .sql import select, prepare_splitfile_sql, POSTGRES_MAX_IDENTIFIER
 from .table import Table
@@ -176,6 +174,10 @@ class Image(NamedTuple):
         # assumes that we got to the point in the normal checkout where we're about to materialize the tables
         # (e.g. the schemata are cleared)
         # Use a per-schema "foreign server" for layered queries for now
+
+        # Circular import
+        from splitgraph.hooks.data_source.fdw import init_fdw
+
         target_schema = target_schema or self.repository.to_schema()
         server_id = "%s_lq_checkout_server" % target_schema
         engine = self.repository.engine
@@ -207,7 +209,9 @@ class Image(NamedTuple):
             self.get_table(table_name).materialize(table_name, target_schema, lq_server=server_id)
 
     @contextmanager
-    def query_schema(self, wrapper: Optional[str] = FDW_CLASS) -> Iterator[str]:
+    def query_schema(
+        self, wrapper: Optional[str] = FDW_CLASS, commit: bool = True
+    ) -> Iterator[str]:
         """
         Creates a temporary schema with tables in this image mounted as foreign tables that can be accessed via
         read-only layered querying. On exit from the context manager, the schema is discarded.
@@ -218,7 +222,8 @@ class Image(NamedTuple):
         try:
             self.object_engine.create_schema(tmp_schema)
             self._lq_checkout(target_schema=tmp_schema, wrapper=wrapper)
-            self.object_engine.commit()  # Make sure the new tables are seen by other connections
+            if commit:
+                self.object_engine.commit()  # Make sure the new tables are seen by other connections
 
             # Inject extra query planner hints as session variables if specified.
             lq_tuning = get_singleton(CONFIG, "SG_LQ_TUNING")
