@@ -5,6 +5,8 @@ tracking tables for changes and uploading/downloading tables to other remote eng
 By default, Splitgraph is backed by Postgres: see :mod:`splitgraph.engine.postgres` for an example of how to
 implement a different engine.
 """
+import re
+import threading
 from abc import ABC
 from contextlib import contextmanager
 from enum import Enum
@@ -76,6 +78,17 @@ class ResultShape(Enum):
     MANY_MANY = 4  # e.g. [("row1_val1", "row1_val_2"), ("row2_val1", "row2_val_2"), ...]
 
 
+def validate_type(t: str) -> str:
+    if not re.match(r"[\w ()\[\]]", t):
+        raise ValueError("Invalid type %s!" % t)
+    return t
+
+
+class SavepointStack(threading.local):
+    def __init__(self):
+        self.stack: List[str] = []
+
+
 class SQLEngine(ABC):
     """Abstraction for a Splitgraph SQL backend. Requires any overriding classes to implement `run_sql` as well as
     a few other functions. Together with the `information_schema` (part of the SQL standard), this class uses those
@@ -83,7 +96,7 @@ class SQLEngine(ABC):
     and loading tables."""
 
     def __init__(self) -> None:
-        self._savepoint_stack: List[str] = []
+        self._savepoint_stack = SavepointStack()
 
     @contextmanager
     def savepoint(self, name: str) -> Iterator[None]:
@@ -91,15 +104,15 @@ class SQLEngine(ABC):
         error that occurs in run_sql results in a rollback to this savepoint rather than the
         rollback of the whole transaction. At exit, the savepoint is released."""
         self.run_sql(SQL("SAVEPOINT ") + Identifier(name))
-        self._savepoint_stack.append(name)
+        self._savepoint_stack.stack.append(name)
         try:
             yield
             # Don't catch any exceptions here: the implementer's run_sql method is supposed
             # to do that and roll back to the savepoint.
         finally:
             # If the savepoint wasn't rolled back to, release it.
-            if self._savepoint_stack and self._savepoint_stack[-1] == name:
-                self._savepoint_stack.pop()
+            if self._savepoint_stack.stack and self._savepoint_stack.stack[-1] == name:
+                self._savepoint_stack.stack.pop()
                 self.run_sql(SQL("RELEASE SAVEPOINT ") + Identifier(name))
 
     def run_sql(self, statement, arguments=None, return_shape=ResultShape.MANY_MANY, named=False):
@@ -260,8 +273,7 @@ class SQLEngine(ABC):
         )
 
     def get_table_type(self, schema: str, table: str) -> Optional[str]:
-        """Get the type of the table (BASE or FOREIGN)
-        """
+        """Get the type of the table (BASE or FOREIGN)"""
         result = self.run_sql(
             "SELECT table_type FROM information_schema.tables WHERE table_schema = %s"
             " AND table_name = %s",
@@ -309,9 +321,9 @@ class SQLEngine(ABC):
         query = (
             SQL("CREATE " + flavour + " TABLE ")
             + target
-            + SQL(" (" + ",".join("{} %s " % col.pg_type for col in schema_spec)).format(
-                *(Identifier(col.name) for col in schema_spec)
-            )
+            + SQL(
+                " (" + ",".join("{} %s " % validate_type(col.pg_type) for col in schema_spec)
+            ).format(*(Identifier(col.name) for col in schema_spec))
         )
 
         pk_cols = [col.name for col in schema_spec if col.is_pk]
@@ -521,7 +533,9 @@ class ObjectEngine:
         :param object_ids: IDs of objects to delete
         """
 
-    def store_fragment(self, inserted, deleted, schema, table, source_schema, source_table):
+    def store_fragment(
+        self, inserted, deleted, schema, table, source_schema, source_table, source_schema_spec
+    ):
         """
         Store a fragment of a changed table in another table
 
@@ -531,6 +545,7 @@ class ObjectEngine:
         :param table: Table to store the change in
         :param source_schema: Schema the source table is located in
         :param source_table: Name of the source table
+        :param source_schema_spec: Schema of the source table (optional)
         """
         raise NotImplementedError()
 
